import java.util.ArrayList;
import java.util.Scanner;
import java.io.File;
import java.io.FileNotFoundException;

class DataFrame {
    /**
     * Constructor empty frame data
     */
    public DataFrame() {
	names = new ArrayList<String>();
	examples = new ArrayList<Example>();
	values = new ArrayList<AttributeValues>();
    }
	
    /**
     * Constructor from a file
     */
    public DataFrame(String fileName) {
	names = new ArrayList<String>();
	examples = new ArrayList<Example>();
	values = new ArrayList<AttributeValues>();
	readFile(fileName);
    }
	
    /**
     * 
     */
    public void readFile(String fileName) {
	try {
	    File f = new File(fileName);
	    Scanner scanner = new Scanner(f);
	    
	    String name;
			
	    // first line : names of attributes
	    Scanner sc = new Scanner(scanner.nextLine());
	    while(sc.hasNext()) {
		names.add(sc.next());
		AttributeValues v = new AttributeValues();
		values.add(v);
	    }
	    sc.close();
			
	    // examples
	    String v;
	    int id;
	    while(scanner.hasNext()) {
		sc = new Scanner(scanner.nextLine());
		Example example = new Example();
		int j = 0;
		while(sc.hasNext()) {
		    v = sc.next();
		    id = values.get(j).add(v);
		    example.add(id);
		    j++;
		}
		sc.close();
		examples.add(example);
	    }
	    
	    scanner.close();

	} catch (FileNotFoundException exception) {
	    System.out.println("File not found");
	}
    }
	
	
    /**
     * Add an example in the data frame
     */
    public void add(Example ex) {
	examples.add(ex);
    }
	
    /**
     * Number of examples
     */
    public int size() {
	return examples.size();
    }
	
    /**
     * Set the names of attributes
     */
    public void setNames(ArrayList<String> n) {
	names = n;
    }
	
    /**
     * 
     */
    public void setValues(ArrayList<AttributeValues> values) {
	this.values = values;
    }
	
    /**
     * Get the names of the attributes
     */
    public ArrayList<String> getNames() {
	return names;
    }
	
    /**
     * 
     */
    public Example getExample(int i) {
	return examples.get(i);
    }

    public int getAttributeId(String name) {
	for(int i = 0; i < names.size(); i++)
	    if (name.compareTo(names.get(i)) == 0)
		return i;
	return 0;
    }
	
    /**
     * Get the list of examples
     */
    public ArrayList<Example> getExamples() {
	return examples;
    }
	
    /**
     * get the sub data frame according to the selection
     */
    public DataFrame select(int attribute, String value) {
	return select(attribute, values.get(attribute).getId(value));
    }

    /**
     * get the sub data frame according to the selection
     */
    public DataFrame select(int attribute, int valueId) {
	DataFrame df = new DataFrame();
	df.setNames(names);
	df.setValues(values);

	for(Example example : examples)
	    if (example.get(attribute) == valueId)
		df.add(example);
		
	return df;
    }
	
    /**
     * to print the data frame
     */
    public String toString() {
	String res = "";
		
	if (names.size() > 0) {
	    res = names.get(0);
	    for(String name : names)
		res += " " + name;
	    res += "\n";
			
	    for(Example example : examples)
		res += example.toString(this) + "\n";
	}
		
	return res;
    }
	
    // Names of the attributes
    protected ArrayList<String> names;
	
    // List of the Examples
    protected ArrayList<Example> examples;

    // The value names for each attribute
    public ArrayList<AttributeValues> values;

}
